@@ -3,8 +3,17 @@ RustPlus, An API wrapper for interfacing with the Rust+ App API
 """
 
 from .api import RustSocket
-from .api.structures import EntityEvent, TeamEvent, ChatEvent, MarkerEvent, RustMarker
+from .api.remote.events import (
+    EntityEvent,
+    TeamEvent,
+    ChatEvent,
+    MarkerEvent,
+    ProtobufEvent,
+    RegisteredListener,
+)
+from .api.structures import RustMarker
 from .api.remote.fcm_listener import FCMListener
+from .api.remote.ratelimiter import RateLimiter
 from .commands import CommandOptions, Command
 from .exceptions import *
 from .conversation import ConversationFactory, Conversation, ConversationPrompt
@@ -12,5 +21,5 @@ from .utils import *
 
 __name__ = "rustplus"
 __author__ = "olijeffers0n"
-__version__ = "5.4.9"
+__version__ = "5.5.3"
 __support__ = "Discord: https://discord.gg/nQqJe8qvP8"
