@@ -3,14 +3,23 @@ from asyncio.futures import Future
 from typing import List, Callable, Union
 from PIL import Image
 
+from .remote.events.event_loop_manager import EventLoopManager
 from .structures import *
 from .remote.rustplus_proto import AppEmpty, AppRequest
-from .remote import RustRemote, HeartBeat, MapEventListener
+from .remote import RustRemote, HeartBeat, MapEventListener, ServerChecker, RateLimiter
 from ..commands import CommandOptions, CommandHandler
 from ..commands.command_data import CommandData
 from ..exceptions import *
-from ..utils import RegisteredListener, deprecated
+from .remote.events import (
+    RegisteredListener,
+    EntityEvent,
+    TeamEvent,
+    ChatEvent,
+    ProtobufEvent,
+)
+from ..utils import deprecated
 from ..conversation import ConversationFactory
+from ..utils import ServerID
 
 
 class BaseRustSocket:
@@ -27,6 +36,8 @@ class BaseRustSocket:
         heartbeat: HeartBeat = None,
         use_proxy: bool = False,
         use_test_server: bool = False,
+        event_loop: asyncio.AbstractEventLoop = None,
+        rate_limiter: RateLimiter = None,
     ) -> None:
 
         if ip is None:
@@ -36,26 +47,27 @@ class BaseRustSocket:
         if player_token is None:
             raise ValueError("PlayerToken cannot be None")
 
-        self.ip = ip
-        self.port = port
         self.steam_id = steam_id
         self.player_token = player_token
+        self.server_id = ServerID(ip, port, steam_id, player_token)
         self.seq = 1
         self.command_options = command_options
         self.raise_ratelimit_exception = raise_ratelimit_exception
+        self.ratelimit_limit = ratelimit_limit
+        self.ratelimit_refill = ratelimit_refill
         self.marker_listener = MapEventListener(self)
         self.use_test_server = use_test_server
+        self.event_loop = event_loop
 
         self.remote = RustRemote(
-            ip=self.ip,
-            port=self.port,
+            server_id=self.server_id,
             command_options=command_options,
             ratelimit_limit=ratelimit_limit,
             ratelimit_refill=ratelimit_refill,
             use_proxy=use_proxy,
             api=self,
-            loop=asyncio.get_event_loop_policy().get_event_loop(),
             use_test_server=use_test_server,
+            rate_limiter=rate_limiter,
         )
 
         if heartbeat is None:
@@ -71,18 +83,19 @@ class BaseRustSocket:
         """
         while True:
 
-            if self.remote.ratelimiter.can_consume(amount):
-                self.remote.ratelimiter.consume(amount)
-                self.heartbeat.reset_rhythm()
-                return
+            if self.remote.ratelimiter.can_consume(self.server_id, amount):
+                self.remote.ratelimiter.consume(self.server_id, amount)
+                break
 
             if self.raise_ratelimit_exception:
                 raise RateLimitError("Out of tokens")
 
             await asyncio.sleep(
-                self.remote.ratelimiter.get_estimated_delay_time(amount)
+                self.remote.ratelimiter.get_estimated_delay_time(self.server_id, amount)
             )
 
+        self.heartbeat.reset_rhythm()
+
     def _generate_protobuf(self) -> AppRequest:
         """
         Generates the default protobuf for a request
@@ -106,6 +119,23 @@ class BaseRustSocket:
 
         :return: None
         """
+        EventLoopManager.set_loop(
+            self.event_loop
+            if self.event_loop is not None
+            else asyncio.get_event_loop(),
+            self.server_id,
+        )
+
+        if not self.use_test_server:
+            ServerChecker(self.server_id.ip, self.server_id.port).run()
+
+        EventLoopManager.set_loop(
+            self.event_loop
+            if self.event_loop is not None
+            else asyncio.get_event_loop(),
+            self.server_id,
+        )
+
         try:
             if self.remote.ws is None:
                 await self.remote.connect(
@@ -189,8 +219,7 @@ class BaseRustSocket:
         await self.disconnect()
 
         # Reset basic credentials
-        self.ip = ip
-        self.port = port
+        self.server_id = ServerID(ip, port, steam_id, player_token)
         self.steam_id = steam_id
         self.player_token = player_token
         self.seq = 1
@@ -204,7 +233,7 @@ class BaseRustSocket:
                 self.remote.command_handler.command_options = command_options
             else:
                 self.remote.use_commands = True
-                self.remote.command_handler = CommandHandler(self.command_options)
+                self.remote.command_handler = CommandHandler(self.command_options, self)
 
         self.raise_ratelimit_exception = raise_ratelimit_exception
 
@@ -213,10 +242,17 @@ class BaseRustSocket:
         self.remote.use_proxy = use_proxy
 
         # reset ratelimiter
-        self.remote.ratelimiter.reset()
+        self.remote.ratelimiter.remove(self.server_id)
+        self.remote.ratelimiter.add_socket(
+            self.server_id,
+            self.ratelimit_limit,
+            self.ratelimit_limit,
+            1,
+            self.ratelimit_refill,
+        )
         self.remote.conversation_factory = ConversationFactory(self)
         # remove entity events
-        self.remote.event_handler.clear_entity_events()
+        EntityEvent.handlers.unregister_all()
         # reset marker listener
         self.marker_listener.persistent_ids.clear()
         self.marker_listener.highest_id = 0
@@ -245,12 +281,11 @@ class BaseRustSocket:
         if asyncio.iscoroutinefunction(coro):
             cmd_data = CommandData(
                 coro,
-                asyncio.get_event_loop_policy().get_event_loop(),
                 aliases,
                 alias_func,
             )
             self.remote.command_handler.register_command(cmd_data)
-            return RegisteredListener(coro.__name__, (cmd_data.coro, cmd_data.loop))
+            return RegisteredListener(coro.__name__, cmd_data.coro)
 
         def wrap_func(coro):
 
@@ -262,12 +297,11 @@ class BaseRustSocket:
 
             cmd_data = CommandData(
                 coro,
-                asyncio.get_event_loop_policy().get_event_loop(),
                 aliases,
                 alias_func,
             )
             self.remote.command_handler.register_command(cmd_data)
-            return RegisteredListener(coro.__name__, (cmd_data.coro, cmd_data.loop))
+            return RegisteredListener(coro.__name__, cmd_data.coro)
 
         return wrap_func
 
@@ -282,9 +316,8 @@ class BaseRustSocket:
         if isinstance(coro, RegisteredListener):
             coro = coro.get_coro()
 
-        data = (coro, asyncio.get_event_loop_policy().get_event_loop())
-        listener = RegisteredListener("team_changed", data)
-        self.remote.event_handler.register_event(listener)
+        listener = RegisteredListener("team_changed", coro)
+        TeamEvent.handlers.register(listener, self.server_id)
         return listener
 
     def chat_event(self, coro) -> RegisteredListener:
@@ -298,11 +331,8 @@ class BaseRustSocket:
         if isinstance(coro, RegisteredListener):
             coro = coro.get_coro()
 
-        data = (coro, asyncio.get_event_loop_policy().get_event_loop())
-        listener = RegisteredListener("chat_message", data)
-
-        self.remote.event_handler.register_event(listener)
-
+        listener = RegisteredListener("chat_message", coro)
+        ChatEvent.handlers.register(listener, self.server_id)
         return listener
 
     def entity_event(self, eid):
@@ -343,17 +373,19 @@ class BaseRustSocket:
                 if entity_info.response.HasField("error"):
                     raise SmartDeviceRegistrationError("Not Found")
 
-                self.remote.event_handler.register_event(
+                EntityEvent.handlers.register(
                     RegisteredListener(
-                        eid, (coro, loop, entity_info.response.entityInfo.type)
-                    )
+                        eid, (coro, entity_info.response.entityInfo.type)
+                    ),
+                    self.server_id,
                 )
 
-            loop = asyncio.get_event_loop_policy().get_event_loop()
-            future = asyncio.run_coroutine_threadsafe(get_entity(self, eid), loop)
+            future = asyncio.run_coroutine_threadsafe(
+                get_entity(self, eid), EventLoopManager.get_loop(self.server_id)
+            )
             future.add_done_callback(entity_event_callback)
 
-            return RegisteredListener(eid, (coro))
+            return RegisteredListener(eid, coro)
 
         return wrap_func
 
@@ -379,8 +411,7 @@ class BaseRustSocket:
         if not self.marker_listener:
             raise ValueError("Marker listener not started")
 
-        data = (coro, asyncio.get_event_loop_policy().get_event_loop())
-        listener = RegisteredListener("map_marker", data)
+        listener = RegisteredListener("map_marker", coro)
         self.marker_listener.add_listener(listener)
         return listener
 
@@ -395,9 +426,8 @@ class BaseRustSocket:
         if isinstance(coro, RegisteredListener):
             coro = coro.get_coro()
 
-        data = (coro, asyncio.get_event_loop_policy().get_event_loop())
-        listener = RegisteredListener("protobuf_received", data)
-        self.remote.event_handler.register_event(listener)
+        listener = RegisteredListener("protobuf_received", coro)
+        ProtobufEvent.handlers.register(listener, self.server_id)
         return listener
 
     def remove_listener(self, listener) -> bool:
@@ -406,11 +436,26 @@ class BaseRustSocket:
 
         :return: Success of removal. True = Removed. False = Not Removed
         """
-
         if isinstance(listener, RegisteredListener):
             if listener.listener_id == "map_marker":
                 return self.marker_listener.remove_listener(listener)
-            return self.remote.remove_listener(listener)
+
+            if ChatEvent.handlers.has(listener, self.server_id):
+                ChatEvent.handlers.unregister(listener, self.server_id)
+                return True
+
+            if TeamEvent.handlers.has(listener, self.server_id):
+                TeamEvent.handlers.unregister(listener, self.server_id)
+                return True
+
+            if EntityEvent.handlers.has(listener, self.server_id):
+                EntityEvent.handlers.unregister(listener, self.server_id)
+                return True
+
+            if ProtobufEvent.handlers.has(listener, self.server_id):
+                ProtobufEvent.handlers.unregister(listener, self.server_id)
+                return True
+
         return False
 
     @staticmethod
@@ -487,7 +532,7 @@ class BaseRustSocket:
         add_vending_machines: bool = False,
         override_images: dict = None,
         add_grid: bool = False,
-    ) -> Image:
+    ) -> Image.Image:
         """
         Gets an image of the map from the server with the specified additions
 
