@@ -2,11 +2,12 @@ import asyncio
 import logging
 import time
 
-from .event_handler import EventHandler
+from .events.event_handler import EventHandler
 from .rustplus_proto import AppRequest, AppMessage
 from .rustws import RustWebsocket, CONNECTED, PENDING_CONNECTION
-from .token_bucket import RateLimiter
+from .ratelimiter import RateLimiter
 from .expo_bundle_handler import MagicValueGrabber
+from ...utils import ServerID
 from ...conversation import ConversationFactory
 from ...commands import CommandHandler
 from ...exceptions import (
@@ -14,32 +15,34 @@ from ...exceptions import (
     ResponseNotReceivedError,
     RequestError,
 )
-from ...utils import RegisteredListener
 
 
 class RustRemote:
     def __init__(
         self,
-        ip,
-        port,
+        server_id: ServerID,
         command_options,
         ratelimit_limit,
         ratelimit_refill,
         websocket_length=600,
         use_proxy: bool = False,
         api=None,
-        loop=None,
         use_test_server: bool = False,
+        rate_limiter: RateLimiter = None,
     ) -> None:
 
-        self.ip = ip
-        self.port = port
+        self.server_id = server_id
         self.command_options = command_options
         self.ratelimit_limit = ratelimit_limit
         self.ratelimit_refill = ratelimit_refill
         self.use_proxy = use_proxy
-        self.ratelimiter = RateLimiter(
-            ratelimit_limit, ratelimit_limit, 1, ratelimit_refill
+        if isinstance(rate_limiter, RateLimiter):
+            self.ratelimiter = rate_limiter
+        else:
+            self.ratelimiter = RateLimiter.default()
+
+        self.ratelimiter.add_socket(
+            api.server_id, ratelimit_limit, ratelimit_limit, 1, ratelimit_refill
         )
         self.ws = None
         self.websocket_length = websocket_length
@@ -53,27 +56,24 @@ class RustRemote:
             self.use_commands = False
         else:
             self.use_commands = True
-            self.command_handler = CommandHandler(self.command_options)
+            self.command_handler = CommandHandler(self.command_options, api)
 
         self.event_handler = EventHandler()
 
         self.magic_value = MagicValueGrabber.get_magic_value()
         self.conversation_factory = ConversationFactory(api)
-        self.loop = loop
         self.use_test_server = use_test_server
 
     async def connect(self, retries, delay, on_failure=None) -> None:
 
         self.ws = RustWebsocket(
-            ip=self.ip,
-            port=self.port,
+            server_id=self.server_id,
             remote=self,
             use_proxy=self.use_proxy,
             magic_value=self.magic_value,
             use_test_server=self.use_test_server,
             on_failure=on_failure,
             delay=delay,
-            loop=asyncio.get_event_loop_policy().get_event_loop(),
         )
         await self.ws.connect(retries=retries)
 
@@ -146,23 +146,27 @@ class RustRemote:
 
             # Fully Refill the bucket
 
-            self.ratelimiter.last_consumed = time.time()
-            self.ratelimiter.bucket.current = 0
+            self.ratelimiter.socket_buckets.get(self.server_id).current = 0
 
-            while self.ratelimiter.bucket.current < self.ratelimiter.bucket.max:
+            while (
+                self.ratelimiter.socket_buckets.get(self.server_id).current
+                < self.ratelimiter.socket_buckets.get(self.server_id).max
+            ):
                 await asyncio.sleep(1)
-                self.ratelimiter.bucket.refresh()
+                self.ratelimiter.socket_buckets.get(self.server_id).refresh()
 
             # Reattempt the sending with a full bucket
             cost = self.ws.get_proto_cost(app_request)
 
             while True:
 
-                if self.ratelimiter.can_consume(cost):
-                    self.ratelimiter.consume(cost)
+                if self.ratelimiter.can_consume(self.server_id, cost):
+                    self.ratelimiter.consume(self.server_id, cost)
                     break
 
-                await asyncio.sleep(self.ratelimiter.get_estimated_delay_time(cost))
+                await asyncio.sleep(
+                    self.ratelimiter.get_estimated_delay_time(self.server_id, cost)
+                )
 
             await self.send_message(app_request)
             response = await self.get_response(seq, app_request)
@@ -171,16 +175,3 @@ class RustRemote:
             raise RequestError(response.response.error.error)
 
         return response
-
-    def remove_listener(self, listener: RegisteredListener) -> bool:
-
-        if self.event_handler.has_event(listener):
-            self.event_handler.remove_event(listener)
-            return True
-
-        elif self.use_commands:
-            if self.command_handler.has_command(listener):
-                self.command_handler.remove_command(listener)
-                return True
-
-        return False
